from AutoRPE.UtilsRPE.Functions import remove_literals
import AutoRPE.UtilsRPE.RegexPattern as RegexPattern
import AutoRPE.UtilsRPE.Error as Error
import re


class CurrentBlock:
    def __init__(self, _Module_main=None):
        if _Module_main is None:
            self.routine = None
            self.module_name = ""
        else:
            self.routine = _Module_main
            self.module_name = _Module_main.module.name
        self.parent_block = None


# Routine to know if we are entering or exiting a routine
# If the line corresponds to the start of a routine, returns the name of this routine
# If the line corresponds to the end of a routine, returns None
# If is neither one of these cases, returns the same routine
def in_which_block(_line, current_block, vault=None):
    _line = _line.strip().lower()
    _line = remove_literals(_line)
    _pattern_sub = r"subroutine *(\w*) *\(*"
    _pattern_func = r"^[^:]*function\b *(\w+) *\(*"
    _pattern = None
    if _line.count("subroutine"):
        _pattern = _pattern_sub
    elif _line.count("function"):
        _pattern = _pattern_func
    elif _line.find("type") == 0:
        _pattern = RegexPattern.simple_type_declaration
    # We are changing block
    if _pattern is not None:
        # We are exiting a block
        if _line.find("end") == 0:
            # Check we are not exiting a nested function
            if current_block.parent_block is not None:
                if current_block.parent_block == current_block.routine:
                    current_block.parent_block = None
                    current_block.routine = None
                else:
                    current_block.routine = current_block.parent_block
            else:
                current_block.routine = None

        # We are entering a new block
        else:
            match = re.search(_pattern, _line, flags=re.IGNORECASE)
            # cases like variables names that contains "function" in it will not match
            if match is not None:
                block_name = match.groups(0)[0].strip()
                if vault:
                    try:
                        procedure = vault.get_procedure_by_name(block_name, module_name=current_block.module_name)
                        current_block.routine = procedure
                        if current_block.parent_block is None:
                            current_block.parent_block = procedure
                    except Error.ProcedureNotFound:
                        current_block.routine = None
                else:
                    current_block.routine = block_name
    return current_block


def in_which_type_definition(_line, type_def):
    _line = _line.strip().lower()
    _line = remove_literals(_line)
    _pattern_type = "^ *type[^(].*::(.*)"
    _pattern_type_alt = "^ *type[^(](.*)"
    _pattern_type_end = "^ *end *type"

    # We are changing block
    if type_def is None:
        m = re.match(_pattern_type, _line, re.I)
        if m:
            type_name = m.group(1).strip()
            return type_name
        n = re.match(_pattern_type_alt, _line, re.I)
        if n:
            type_name = n.group(1).strip()
            return type_name
    else:
        m = re.search(_pattern_type_end, _line, re.I)
        if m:
            return None
    return type_def